// LSMSDK
//
// Copyright (c) 2013 LifeStreet Media. All rights reserved.
//
// $LastChangedDate: 2013-04-30 11:15:56 +0400 (Tue, 30 Apr 2013) $
// @version: $LastChangedRevision: 91879 $

var mraid = (function() {
    var VERSION = "1.0";

    var STATE = {
        LOADING: "loading",
        DEFAULT: "default",
        EXPANDED: "expanded",
        HIDDEN: "hidden"
    };

    var PLACEMENT_TYPE = {
        INLINE: "inline",
        INTERSTITIAL: "interstitial"
    };

    var EVENT = {
        READY: "ready", // function()
        ERROR: "error", // function(String: message, String: action)
        STATE_CHANGE: "stateChange", // function(String: state)
        VIEWABLE_CHANGE: "viewableChange" // function(boolean)
    };

    var _state = STATE.LOADING;
    var _viewable = false;
    var _placementType = PLACEMENT_TYPE.INLINE;
    var _events = [];

    var _expandProperties = {
        "width": 0,
        "height": 0,
        "useCustomClose": false,
        "isModal": true // read only
    };

    function _callNativeMethod(method, params) {
        MRAIDNativeInterface.callMethod(method, JSON.stringify(params));
    }

    function _fireEvent(event) {
        var args = Array.prototype.slice.call(arguments);
        args.shift();
        for (var i = 0; i < _events.length; i++) {
            if (_events[i].event == event && typeof _events[i].listener === "function") {
                _events[i].listener.apply(null, args);
            }
        }
    }

    function _setState(newState) {
        if (_state != newState) {
            _state = newState;
            _fireEvent(EVENT.STATE_CHANGE, newState);
        }
    }

    function _setViewable(viewable) {
        if (_viewable != viewable) {
            _viewable = viewable;
            _fireEvent(EVENT.VIEWABLE_CHANGE, viewable);
        }
    }

    //window.addEventListener("load", function(e){
    _callNativeMethod("onLoad", {});
    //});

    return {
        _bridge: (function() {
            return {
                ready: function() {
                    _setState(STATE.DEFAULT);
                    _fireEvent(EVENT.READY);
                },
                expandedState: function() {
                    _setState(STATE.EXPANDED);
                },
                defaultState: function() {
                    _setState(STATE.DEFAULT);
                },
                hiddenState: function() {
                    _setState(STATE.HIDDEN);
                    _setViewable(false);
                },
                viewable: function() {
                    _setViewable(true);
                },
                notViewable: function() {
                    _setViewable(false);
                },
                reportError: function(msg, action) {
                    _fireEvent(EVENT.ERROR, msg, action);
                },
                placementTypeInline: function() {
                    _placementType = PLACEMENT_TYPE.INLINE;
                },
                placementTypeInterstitial: function() {
                    _placementType = PLACEMENT_TYPE.INTERSTITIAL;
                }

            };
        })(),

        getVersion: function() {
            return VERSION;
        },

        addEventListener: function(event, listener) {
            if (!event || !listener) {
                return;
            }

            _events.push({
                "event": event,
                "listener": listener
            });
        },

        removeEventListener: function(event, listener) {
            if (!event) {
                return;
            }

            for (var i = 0; i < _events.length; i++) {
                if (_events[i]["event"] == event && (!listener || _events[i]["listener"] == listener)) {
                    _events.splice(i, 1);
                    i--;
                }
            }
        },

        getState: function() {
            return _state;
        },

        getPlacementType: function() {
            return _placementType;
        },

        isViewable: function() {
            return _viewable;
        },

        expand: function(url) {
            if (_state != STATE.DEFAULT) {
                return;
            }

            var args = {
                "width": (_expandProperties["width"]).toString(),
                "height": (_expandProperties["height"]).toString(),
                "useCustomClose": (_expandProperties["useCustomClose"]).toString()
            };

            if (url) {
                args["url"] = (url).toString();
            }

            _callNativeMethod("expand", args);
        },

        getExpandProperties: function() {
            var result = {};
            for (var property in _expandProperties) {
                result[property] = _expandProperties[property];
            }
            return result;
        },

        setExpandProperties: function(properties) {
            for (property in properties) {
                if (property != "isModal" && property in _expandProperties) {
                    _expandProperties[property] = properties[property];
                }
            }
        },

        close: function() {
            if (_state != STATE.HIDDEN) {
                _callNativeMethod("close", {
                    "state": _state
                });
            }
        },

        useCustomClose: function(customClose) {
            _expandProperties["useCustomClose"] = Boolean(customClose);
            _callNativeMethod("useCustomClose", {
                "customClose": Boolean(customClose).toString()
            });
        },

        open: function(url) {
            if (url) {
                _callNativeMethod("open", {
                    "url": url.toString()
                });
            }
        }
    };
})();
